@@ -71,20 +71,27 @@ class Manager(manager.Manager):
         super(Manager, self).__init__(credentials=credentials)
 
         self.images_client = images_client.ImagesClient(
-            self.auth_provider, 'image', CONF.identity.region)
+            self.auth_provider, 'image', CONF.identity.region,
+            disable_ssl_certificate_validation=True)
         self.ports_client = ports_client.PortsClient(
-            self.auth_provider, 'network', CONF.identity.region)
+            self.auth_provider, 'network', CONF.identity.region,
+            disable_ssl_certificate_validation=True)
         self.sgs_client = security_groups_client.SecurityGroupsClient(
-            self.auth_provider, 'network', CONF.identity.region)
+            self.auth_provider, 'network', CONF.identity.region,
+            disable_ssl_certificate_validation=True)
         self.vol_client = volumes_client.VolumesClient(
-            self.auth_provider, 'volumev3', CONF.identity.region)
+            self.auth_provider, 'volumev3', CONF.identity.region,
+            disable_ssl_certificate_validation=True)
         self.container_client = ZunClient(self.auth_provider)
         self.neutron_client = networks_client.NetworksClient(
-            self.auth_provider, 'network', CONF.identity.region)
+            self.auth_provider, 'network', CONF.identity.region,
+            disable_ssl_certificate_validation=True)
         self.subnets_client = subnets_client.SubnetsClient(
-            self.auth_provider, 'network', CONF.identity.region)
+            self.auth_provider, 'network', CONF.identity.region,
+            disable_ssl_certificate_validation=True)
         self.subnetpools_client = subnetpools_client.SubnetpoolsClient(
-            self.auth_provider, 'network', CONF.identity.region)
+            self.auth_provider, 'network', CONF.identity.region,
+            disable_ssl_certificate_validation=True)
 
 
 class ZunClient(rest_client.RestClient):
@@ -146,6 +153,11 @@ class ZunClient(rest_client.RestClient):
 
         return url
 
+    def container_action_uri(cls, container_id, request_id):
+        url = "/containers/{0}/container_actions/{1}".format(
+            container_id, request_id)
+        return url
+
     @classmethod
     def add_params(cls, url, params):
         """add_params adds dict values (params) to url as query parameters
@@ -251,6 +263,10 @@ class ZunClient(rest_client.RestClient):
         return self.post(
             self.container_uri(container_id, action='reboot'), None, **kwargs)
 
+    def rebuild_container(self, container_id, **kwargs):
+        return self.post(
+            self.container_uri(container_id, action='rebuild'), None, **kwargs)
+
     def exec_container(self, container_id, command, **kwargs):
         return self.post(
             self.container_uri(container_id, action='execute'),
@@ -294,6 +310,12 @@ class ZunClient(rest_client.RestClient):
         return self.deserialize(resp, body,
                                 service_model.ServiceCollection)
 
+    def get_container_action(self, container_id, request_id):
+        resp, body = self.get(
+            self.container_action_uri(container_id, request_id))
+        return self.deserialize(
+            resp, body, container_model.ContainerActionEntity)
+
     def ensure_container_in_desired_state(self, container_id, status):
         def is_container_in_desired_state():
             _, container = self.get_container(container_id)
@@ -313,6 +335,15 @@ class ZunClient(rest_client.RestClient):
                 return True
         utils.wait_for_condition(is_container_deleted)
 
+    def ensure_action_finished(self, container_id, request_id):
+        def is_action_finished():
+            _, action = self.get_container_action(container_id, request_id)
+            if action.finish_time is not None:
+                return True
+            else:
+                return False
+        utils.wait_for_condition(is_action_finished, timeout=120)
+
     def network_attach(self, container_id, params=None, **kwargs):
         return self.post(
             self.container_uri(container_id, action='network_attach',
@@ -398,46 +429,9 @@ class DockerHTTPClient(docker.APIClient):
             tls=False
         )
 
-    def list_instances(self, inspect=False):
-        """List all containers."""
-        res = []
-        for container in self.containers(all=True):
-            info = self.inspect_container(container['Id'])
-            if not info:
-                continue
-            if inspect:
-                res.append(info)
-            else:
-                res.append(info['Config'].get('Hostname'))
-        return res
-
-    def list_containers(self):
-        return self.containers(all=True, filters={'name': 'zun-'})
-
 
 class DockerClient(object):
 
-    def get_container(self, container_id,
-                      docker_auth_url=CONF.docker.api_url):
-        with docker_client(docker_auth_url) as docker:
-            for info in docker.list_instances(inspect=True):
-                if container_id in info['Name']:
-                    return info
-            return None
-
-    def ensure_container_pid_changed(
-            self, container_id, pid,
-            docker_auth_url=CONF.docker.api_url):
-        def is_pid_changed():
-            container = self.get_container(container_id,
-                                           docker_auth_url=docker_auth_url)
-            new_pid = container.get('State').get('Pid')
-            if pid != new_pid:
-                return True
-            else:
-                return False
-        utils.wait_for_condition(is_pid_changed)
-
     def pull_image(
             self, repo, tag=None,
             docker_auth_url=CONF.docker.api_url):
@@ -461,8 +455,3 @@ class DockerClient(object):
                        docker_auth_url=CONF.docker.api_url):
         with docker_client(docker_auth_url) as docker:
             return docker.remove_network(name)
-
-    def get_archive(self, container_id, path,
-                    docker_auth_url=CONF.docker.api_url):
-        with docker_client(docker_auth_url) as docker:
-            return docker.get_archive(container_id, path)
