@@ -12,12 +12,14 @@
 
 from io import BytesIO
 import tarfile
+import testtools
 import time
 import types
 
 from oslo_serialization import jsonutils as json
 from oslo_utils import encodeutils
 import six
+from tempest import config
 from tempest.lib.common.utils import data_utils
 from tempest.lib import decorators
 from tempest.lib import exceptions as lib_exc
@@ -28,6 +30,9 @@ from zun_tempest_plugin.tests.tempest import base
 from zun_tempest_plugin.tests.tempest import utils
 
 
+CONF = config.CONF
+
+
 class TestContainer(base.BaseZunTest):
 
     credentials = ['primary', 'admin']
@@ -48,7 +53,6 @@ class TestContainer(base.BaseZunTest):
         super(TestContainer, cls).setup_clients()
         cls.images_client = cls.os_primary.images_client
         cls.sgs_client = cls.os_primary.sgs_client
-        cls.vol_client = cls.os_primary.vol_client
 
     @classmethod
     def resource_setup(cls):
@@ -97,17 +101,23 @@ class TestContainer(base.BaseZunTest):
     @decorators.idempotent_id('cef53a56-22b7-4808-b01c-06b2b7126115')
     def test_delete_container(self):
         _, container = self._create_container()
-        docker_url = self._get_docker_url(container)
         resp, _ = self.container_client.delete_container(container.uuid)
         self.assertEqual(204, resp.status)
         self.container_client.ensure_container_deleted(container.uuid)
-        container = self.docker_client.get_container(
-            container.uuid, docker_url)
-        self.assertIsNone(container)
 
     @decorators.idempotent_id('ef69c9e7-0ce0-4e14-b7ec-c1dc581a3927')
     def test_run_container(self):
-        self._run_container()
+        _, model = self._run_container(
+            environment={'key1': 'env1', 'key2': 'env2'},
+            labels={'key1': 'label1', 'key2': 'label2'},
+            restart_policy={'Name': 'on-failure', 'MaximumRetryCount': 2},
+        )
+        resp, model = self.container_client.get_container(model.uuid)
+        self.assertEqual(200, resp.status)
+        self.assertEqual({'key1': 'env1', 'key2': 'env2'}, model.environment)
+        self.assertEqual({'key1': 'label1', 'key2': 'label2'}, model.labels)
+        self.assertEqual({'Name': 'on-failure', 'MaximumRetryCount': '2'},
+                         model.restart_policy)
 
     @decorators.idempotent_id('a2152d78-b6a6-4f47-8767-d83d29c6fb19')
     def test_run_container_with_minimal_params(self):
@@ -116,6 +126,9 @@ class TestContainer(base.BaseZunTest):
 
     @decorators.idempotent_id('c32f93e3-da88-4c13-be38-25d2e662a28e')
     def test_run_container_with_image_driver_glance(self):
+        if not CONF.service_available.glance:
+            raise self.skipException("This test requires glance service")
+
         docker_base_url = self._get_docker_url()
         self.docker_client.pull_image(
             'cirros', docker_auth_url=docker_base_url)
@@ -139,29 +152,6 @@ class TestContainer(base.BaseZunTest):
         _, model = self._run_container(
             image='cirros', image_driver='glance')
 
-    @decorators.idempotent_id('b70bedbc-5ba2-400c-8f5f-0cf05ca17151')
-    def test_run_container_with_environment(self):
-        _, model = self._run_container(environment={
-            'key1': 'env1', 'key2': 'env2'})
-
-        container = self.docker_client.get_container(
-            model.uuid,
-            self._get_docker_url(model))
-        env = container.get('Config').get('Env')
-        self.assertTrue('key1=env1' in env)
-        self.assertTrue('key2=env2' in env)
-
-    @decorators.idempotent_id('0e59d549-58ff-440f-8704-10e223c31cbc')
-    def test_run_container_with_labels(self):
-        _, model = self._run_container(labels={
-            'key1': 'label1', 'key2': 'label2'})
-
-        container = self.docker_client.get_container(
-            model.uuid,
-            self._get_docker_url(model))
-        labels = container.get('Config').get('Labels')
-        self.assertEqual({'key1': 'label1', 'key2': 'label2'}, labels)
-
     @decorators.idempotent_id('8fc7fec1-e1a2-3f65-a5a6-dba425c1607c')
     def test_run_container_with_port(self):
         project_id = self.container_client.tenant_id
@@ -173,9 +163,18 @@ class TestContainer(base.BaseZunTest):
         else:
             self.fail('Cannot find network in tenant.')
 
-        port = self.create_port(tenant_network)
+        port_name = data_utils.rand_name('port')
+        port = self.create_port(tenant_network, name=port_name)
         port_address = port['fixed_ips'][0]['ip_address']
         port_subnet = port['fixed_ips'][0]['subnet_id']
+        # NOTE(hongbin): port name will change in Rocky or earlier version
+        # self.assertEqual(port_name, port['name'])
+        self.assertEqual('', port['device_owner'])
+        self.assertEqual('', port['device_id'])
+        self.assertEqual([], port['tags'])
+        port = self.os_admin.ports_client.show_port(port['id'])['port']
+        self.assertEqual('', port['binding:host_id'])
+        self.assertEqual('unbound', port['binding:vif_type'])
 
         _, model = self._run_container(nets=[{'port': port['id']}])
         address = {'port': port['id'],
@@ -183,6 +182,29 @@ class TestContainer(base.BaseZunTest):
                    'subnet_id': port_subnet}
         self._assert_container_has_address(model, address,
                                            network_id=tenant_network['id'])
+        port = self.os_admin.ports_client.show_port(port['id'])['port']
+        # NOTE(hongbin): port name will change in Rocky or earlier version
+        # self.assertEqual(port_name, port['name'])
+        self.assertTrue(port['device_owner'])
+        self.assertEqual(model.uuid, port['device_id'])
+        self.assertTrue(port['tags'])
+        self.assertTrue(port['binding:host_id'])
+        self.assertNotEqual('unbound', port['binding:vif_type'])
+
+        resp, _ = self.container_client.delete_container(
+            model.uuid, params={'stop': True})
+        self.assertEqual(204, resp.status)
+        self.container_client.ensure_container_deleted(model.uuid)
+        port = self.os_admin.ports_client.show_port(port['id'])['port']
+        # NOTE(hongbin): port name will change in Rocky or earlier version
+        # self.assertEqual(port_name, port['name'])
+        self.assertEqual('', port['device_owner'])
+        self.assertEqual('', port['device_id'])
+        # TODO(hongbin): The tags are not always cleanup due to bug #1776035
+        # Need to investigate the issue.
+        # self.assertEqual([], port['tags'])
+        self.assertEqual('', port['binding:host_id'])
+        self.assertEqual('unbound', port['binding:vif_type'])
 
     @decorators.idempotent_id('cfa24356-30fd-42b7-92c7-bbf01bcaf6eb')
     def test_run_container_with_port_in_dual_net(self):
@@ -270,35 +292,10 @@ class TestContainer(base.BaseZunTest):
         subnet = self.create_subnet(network, **subnet_kwargs)
         return pool_id, subnet
 
-    @decorators.idempotent_id('9fc7fec0-e1a9-4f65-a5a6-dba425c1607c')
-    def test_run_container_with_restart_policy(self):
-        _, model = self._run_container(restart_policy={
-            'Name': 'on-failure', 'MaximumRetryCount': 2})
-
-        container = self.docker_client.get_container(
-            model.uuid,
-            self._get_docker_url(model))
-        policy = container.get('HostConfig').get('RestartPolicy')
-        self.assertEqual('on-failure', policy['Name'])
-        self.assertEqual(2, policy['MaximumRetryCount'])
-
-    @decorators.idempotent_id('58585a4f-cdce-4dbd-9741-4416d1098f94')
-    def test_run_container_with_interactive(self):
-        _, model = self._run_container(interactive=True)
-
-        container = self.docker_client.get_container(
-            model.uuid,
-            self._get_docker_url(model))
-        tty = container.get('Config').get('Tty')
-        stdin_open = container.get('Config').get('OpenStdin')
-        self.assertIs(True, tty)
-        self.assertIs(True, stdin_open)
-
     @decorators.idempotent_id('f181eeda-a9d1-4b2e-9746-d6634ca81e2f')
     @utils.requires_microversion('1.20')
     def test_run_container_without_security_groups(self):
         gen_model = datagen.container_data()
-        delattr(gen_model, 'security_groups')
         _, model = self._run_container(gen_model=gen_model)
         sgs = self._get_all_security_groups(model)
         self.assertEqual(1, len(sgs))
@@ -413,6 +410,48 @@ class TestContainer(base.BaseZunTest):
         self.assertEqual(200, resp.status)
         self.assertTrue('hello' in encodeutils.safe_decode(body))
 
+    @decorators.idempotent_id('df7b2518-f779-43f6-b188-28cf3595e251')
+    @utils.requires_microversion('1.24')
+    def test_container_expose_port(self):
+        gen_model = datagen.container_data({'image': 'nginx',
+                                            'exposed_ports': {"80/tcp": {}}})
+        _, model = self._run_container(gen_model=gen_model)
+        # assert security group is created with port 80 open
+        secgroups = model.security_groups
+        self.assertEqual(1, len(secgroups))
+        secgroup = self.sgs_client.show_security_group(secgroups[0])
+        self.assertNotEqual('default', secgroup['security_group']['name'])
+        rules = secgroup['security_group']['security_group_rules']
+        for rule in rules:
+            if (rule['protocol'] == 'tcp' and rule['port_range_min'] == 80 and
+                    rule['port_range_max'] == 80):
+                break
+        else:
+            self.fail('Security group doesnot have rules for opening the port')
+
+        # access the container port
+        ip_address = None
+        for net_id in model.addresses:
+            for address in model.addresses[net_id]:
+                ip_address = address['addr']
+                break
+        self.assertIsNotNone(ip_address)
+        _, m = self._run_container(desired_state='Stopped',
+                                   command=['curl', ip_address])
+        resp, body = self.container_client.logs_container(m.uuid)
+        self.assertEqual(200, resp.status)
+        self.assertTrue(
+            'If you see this page, the nginx web server is successfully '
+            'installed' in encodeutils.safe_decode(body))
+
+        # delete the container and ensure security group is clean up
+        self.container_client.delete_container(
+            model.uuid, params={'stop': True})
+        self.container_client.ensure_container_deleted(model.uuid)
+        self.assertRaises(lib_exc.NotFound,
+                          self.sgs_client.show_security_group,
+                          secgroup['security_group']['id'])
+
     @decorators.idempotent_id('e49231b2-b095-40d3-9b54-33bb1b371cbe')
     @utils.requires_microversion('1.20')
     def test_run_container_with_cinder_volume_dynamic_created(self):
@@ -490,6 +529,9 @@ class TestContainer(base.BaseZunTest):
         4. Create another container from the snapshot image
         5. Verify the pre-created file is there
         """
+        if not CONF.service_available.glance:
+            raise self.skipException("This test requires glance service")
+
         # This command creates a file inside the container
         command = ["/bin/sh", "-c", "echo hello > testfile;sleep 1000000"]
         _, model = self._run_container(command=command)
@@ -538,15 +580,11 @@ class TestContainer(base.BaseZunTest):
         self.assertEqual(202, resp.status)
         self.container_client.ensure_container_in_desired_state(
             model.uuid, 'Stopped')
-        self.assertEqual('Stopped',
-                         self._get_container_state(model))
 
         resp, _ = self.container_client.start_container(model.uuid)
         self.assertEqual(202, resp.status)
         self.container_client.ensure_container_in_desired_state(
             model.uuid, 'Running')
-        self.assertEqual('Running',
-                         self._get_container_state(model))
 
     @decorators.idempotent_id('b5f39756-8898-4e0e-a48b-dda0a06b66b6')
     def test_pause_unpause_container(self):
@@ -556,15 +594,11 @@ class TestContainer(base.BaseZunTest):
         self.assertEqual(202, resp.status)
         self.container_client.ensure_container_in_desired_state(
             model.uuid, 'Paused')
-        self.assertEqual('Paused',
-                         self._get_container_state(model))
 
         resp, _ = self.container_client.unpause_container(model.uuid)
         self.assertEqual(202, resp.status)
         self.container_client.ensure_container_in_desired_state(
             model.uuid, 'Running')
-        self.assertEqual('Running',
-                         self._get_container_state(model))
 
     @decorators.idempotent_id('6179a588-3d48-4372-9599-f228411d1449')
     def test_kill_container(self):
@@ -574,27 +608,35 @@ class TestContainer(base.BaseZunTest):
         self.assertEqual(202, resp.status)
         self.container_client.ensure_container_in_desired_state(
             model.uuid, 'Stopped')
-        self.assertEqual('Stopped',
-                         self._get_container_state(model))
 
     @decorators.idempotent_id('c2e54321-0a70-4331-ba62-9dcaa75ac250')
+    @testtools.skip('temporarily disabled')
     def test_reboot_container(self):
         _, model = self._run_container()
-        docker_base_url = self._get_docker_url(model)
-        container = self.docker_client.get_container(model.uuid,
-                                                     docker_base_url)
-        pid = container.get('State').get('Pid')
 
         resp, _ = self.container_client.reboot_container(model.uuid)
         self.assertEqual(202, resp.status)
-        self.docker_client.ensure_container_pid_changed(model.uuid, pid,
-                                                        docker_base_url)
-        self.assertEqual('Running',
-                         self._get_container_state(model))
-        # assert pid is changed
-        container = self.docker_client.get_container(model.uuid,
-                                                     docker_base_url)
-        self.assertNotEqual(pid, container.get('State').get('Pid'))
+        # TODO(hongbin): wait for reboot to complete and assure it succeeds
+
+    @decorators.idempotent_id('a0c8843f-c32e-4658-b228-eb16c746f495')
+    @utils.requires_microversion('1.33')
+    def test_rebuild_container(self):
+        _, model = self._run_container()
+
+        resp, _ = self.container_client.rebuild_container(model.uuid)
+        self.assertEqual(202, resp.status)
+        request_id = self._get_request_id(resp)
+        # Wait for container to rebuild
+        self.container_client.ensure_action_finished(
+            model.uuid, request_id)
+
+        resp, action = self.container_client.get_container_action(
+            model.uuid, request_id)
+        self.assertEqual(200, resp.status)
+        # if the action succeeds, action.message will be None
+        self.assertIsNone(action.message)
+        self.container_client.ensure_container_in_desired_state(
+            model.uuid, 'Running')
 
     @decorators.idempotent_id('8a591ff8-6793-427f-82a6-e3921d8b4f81')
     def test_exec_container(self):
@@ -618,10 +660,6 @@ class TestContainer(base.BaseZunTest):
         _, model = self._run_container(cpu=0.1, memory=100)
         self.assertEqual('100', model.memory)
         self.assertEqual(0.1, model.cpu)
-        docker_base_url = self._get_docker_url(model)
-        container = self.docker_client.get_container(model.uuid,
-                                                     docker_base_url)
-        self._assert_resource_constraints(container, cpu=0.1, memory=100)
 
         gen_model = datagen.container_patch_data(cpu=0.2, memory=200)
         resp, model = self.container_client.update_container(model.uuid,
@@ -629,9 +667,6 @@ class TestContainer(base.BaseZunTest):
         self.assertEqual(200, resp.status)
         self.assertEqual('200', model.memory)
         self.assertEqual(0.2, model.cpu)
-        container = self.docker_client.get_container(model.uuid,
-                                                     docker_base_url)
-        self._assert_resource_constraints(container, cpu=0.2, memory=200)
 
     @decorators.idempotent_id('b218bea7-f19b-499f-9819-c7021ffc59f4')
     @utils.requires_microversion('1.14')
@@ -690,10 +725,10 @@ class TestContainer(base.BaseZunTest):
         resp, model = self.container_client.get_container(model.uuid)
         self.assertEqual(200, resp.status)
         self.assertEqual('Created', model.status)
-        self.assertEqual('Created', self._get_container_state(model))
         return resp, model
 
-    def _run_container(self, gen_model=None, **kwargs):
+    def _run_container(self, gen_model=None, desired_state='Running',
+                       **kwargs):
         if gen_model is None:
             gen_model = datagen.container_data(**kwargs)
         resp, model = self.container_client.run_container(gen_model)
@@ -701,29 +736,13 @@ class TestContainer(base.BaseZunTest):
         self.assertEqual(202, resp.status)
         # Wait for container to started
         self.container_client.ensure_container_in_desired_state(
-            model.uuid, 'Running')
+            model.uuid, desired_state)
 
         # Assert the container is started
         resp, model = self.container_client.get_container(model.uuid)
-        self.assertEqual('Running', model.status)
-        self.assertEqual('Running', self._get_container_state(model))
+        self.assertEqual(desired_state, model.status)
         return resp, model
 
-    def _get_container_state(self, model):
-        container = self.docker_client.get_container(
-            model.uuid, self._get_docker_url(model))
-        status = container.get('State')
-        if status.get('Error') is True:
-            return 'Error'
-        elif status.get('Paused'):
-            return 'Paused'
-        elif status.get('Running'):
-            return 'Running'
-        elif status.get('Status') == 'created':
-            return 'Created'
-        else:
-            return 'Stopped'
-
     def _get_all_security_groups(self, container):
         # find all neutron ports of this container
         port_ids = set()
@@ -746,14 +765,9 @@ class TestContainer(base.BaseZunTest):
 
         return sg_names
 
-    def _get_docker_url(self, container=None, host='localhost'):
+    def _get_docker_url(self, host='localhost'):
         protocol = 'tcp'
         port = '2375'
-        if container:
-            if not hasattr(container, 'host'):
-                _, container = self.os_admin.container_client.get_container(
-                    container.uuid, params={'all_projects': True})
-            host = container.host
         # NOTE(kiennt): By default, devstack-plugin-container will
         #               set docker_api_url = {
         #                       "unix://$DOCKER_ENGINE_SOCKET_FILE",
@@ -895,7 +909,6 @@ class TestContainerLegacy(TestContainer):
         # Assert the container is started
         resp, model = self.container_client.get_container(model.uuid)
         self.assertEqual('Running', model.status)
-        self.assertEqual('Running', self._get_container_state(model))
         return resp, model
 
     def _create_container(self, **kwargs):
@@ -914,5 +927,4 @@ class TestContainerLegacy(TestContainer):
         resp, model = self.container_client.get_container(model.uuid)
         self.assertEqual(200, resp.status)
         self.assertEqual('Created', model.status)
-        self.assertEqual('Created', self._get_container_state(model))
         return resp, model
